#include "h1jet.h"
#include <cmath>


const bounded_vector<lorentzvector<double> >&
h1jet::operator()(const event_dis& ev, double r0)
{
  int imin, jmin, kmin, nt = ev.upper();
  double tmp, pmin, smin, r2 = r0*r0;
  
  //----- initialize the arrays -----
  _M_pj.resize(1,0); _M_p.resize(1, nt);
  for(int ip = 1; ip <= nt; ip++)
    _M_p[ip] = ev[ip];
  
  for(int n = nt; n > 1; n--) {
    //----- find the smallest resolution variables -----
    imin = 1; jmin = 2; kmin = 1;
    pmin = _M_pair(1,2); smin = _M_p[1].pt2;
    for(int i = 1; i <= n; i++) {
      if((tmp = _M_p[i].pt2) < smin) {
	smin = tmp;
	kmin = i;
      }
      
      for(int j = i + 1; j <= n; j++) 
	if((tmp = _M_pair(i, j)) < pmin) {
	  pmin = tmp;
	  imin = i;
	  jmin = j;
	}
    }
    
    //----- do the clustering -----
    if(smin*r2 < pmin) {
      _M_pj.push_back(_Vec_to_Lv(_M_p[kmin]));
      if(kmin != n) _M_p[kmin] = _M_p[n];
    } else {
      _M_merge(imin, jmin);
      if(jmin != n) _M_p[jmin] = _M_p[n];
    }
  }
  
  //----- add the last (pseudo)particle to the list of jets ----- 
  _M_pj.push_back(_Vec_to_Lv(_M_p[1]));

  return _M_pj;
}

double h1jet::_M_pair(int i, int j)
{
  static const double pi = 3.14159265358979323846;
  static const double twopi = 6.28318530717958647692;
  
  double et2 = (_M_p[i].pt2 < _M_p[j].pt2 ? _M_p[i].pt2 : _M_p[j].pt2);
  double deta = _M_p[i].eta - _M_p[j].eta;
  double dphi = _M_p[i].phi - _M_p[j].phi;
  
  if(dphi >= pi) dphi = fmod(pi+dphi, twopi) - pi;
  else if(dphi < -pi) dphi = -fmod(pi-dphi, twopi) + pi;

  return et2*(deta*deta+dphi*dphi);
}

void h1jet::_M_merge(int i, int j)
{
  _M_p[i].eta *= _M_p[i].pt;  
  _M_p[i].eta += _M_p[j].eta*_M_p[j].pt;  

  _M_p[i].phi *= _M_p[i].pt;  
  _M_p[i].phi += _M_p[j].phi*_M_p[j].pt;  
    
  _M_p[i].pt += _M_p[j].pt;
  _M_p[i].pt2 = _M_p[i].pt*_M_p[i].pt;
  
  _M_p[i].eta /= _M_p[i].pt;
  _M_p[i].phi /= _M_p[i].pt;
}





#ifdef __TEST_H1JET__
#include <phasespace.h>
#include <iostream>


int main(int argc, char **argv)
{
  int n = atoi(argv[1]);
  bool again = false;
  event_dis p(n);
  genevent_dis ps(27, 820);
  h1jet jetclus;
  unsigned long int count = 0;
  
  for(unsigned long int i = 0; i < (unsigned long int) atoi(argv[2]); i++) {
    do {
      again = false;
      try {
	ps(p);
      } catch(...) {
	again = true;
      }
    } while(again);
    
    const bounded_vector<lorentzvector<double> >& jet = jetclus(p);
    
    if(jet.upper() == n-1) {
      count++;
      //for(int j = 1; j <= jet.upper(); j++)
      //cout<<jet[j]<<"  "<<jet[j].mag2()<<endl;
      //cout<<endl;
    }
  }
  cout<<count<<endl;
  return 0;
}
#endif
